use super::*;

/// Trigger a CI run.
#[derive(Parser)]
pub struct TriggerCmd {
    /// Set the node where the node originated from.
    #[clap(long)]
    node: Option<NodeId>,

    /// Set the repository the event refers to. Can be a RID, or the
    /// repository name.
    #[clap(long)]
    repo: String,

    /// Set the name of the ref the event refers to.
    #[clap(long = "ref", aliases = ["name"], default_value = "main")]
    name: String,

    /// Set the commit the event refers to. Can be the SHA1 commit id,
    /// or a symbolic Git revision, as understood by `git rev-parse`.
    /// For example, `HEAD`.
    #[clap(long, default_value = "HEAD")]
    commit: String,

    /// Write the event ID to this file, after adding the event to the
    /// queue.
    #[clap(long)]
    id_file: Option<PathBuf>,

    /// Output the event to trigger a CI run to the standard output,
    /// instead of adding to the event queue in the database.
    #[clap(long)]
    stdout: bool,

    /// Output the event to trigger a CI run to a named file, instead
    /// of adding to the event queue in the database.
    #[clap(long)]
    output: Option<PathBuf>,
}

impl Leaf for TriggerCmd {
    fn run(&self, args: &Args) -> Result<(), CibToolError> {
        let profile = util::load_profile()?;
        let nid = util::lookup_nid(&profile)?;
        let nid = self.node.unwrap_or(nid);
        let (rid, _repo_name) = util::lookup_repo(&profile, &self.repo)?;
        let oid = util::oid_from_cli_arg(&profile, rid, &self.commit)?;

        let base = util::lookup_commit(&profile, rid, &format!("{oid}^")).unwrap_or(oid);
        let name = format!("refs/namespaces/{nid}/refs/heads/{}", self.name.as_str());
        let name =
            RefString::try_from(name.clone()).map_err(|e| CibToolError::RefString(name, e))?;
        let event =
            CiEvent::branch_updated(nid, rid, &name, oid, base).map_err(CibToolError::CiEvent)?;

        if self.stdout {
            let json = event.to_pretty_json().map_err(CibToolError::EventToJson2)?;
            println!("{json}");
        } else if let Some(filename) = &self.output {
            let json = event.to_pretty_json().map_err(CibToolError::EventToJson2)?;
            std::fs::write(filename, &json)
                .map_err(|err| CibToolError::Write(filename.into(), err))?;
        } else {
            let db = args.open_db()?;
            let id = db.push_queued_ci_event(event)?;
            println!("{id}");

            if let Some(filename) = &self.id_file {
                write(filename, id.to_string().as_bytes())
                    .map_err(|e| CibToolError::WriteEventId(filename.into(), e))?;
            }
        }

        Ok(())
    }
}
